StayUnsheathed = LibStub("AceAddon-3.0"):NewAddon("TimerTest", "AceTimer-3.0", "AceConsole-3.0")

function StayUnsheathed:OnInitialize()  
  local defaults = {
    char = {
      EnabledState = true,
      Primary = true,
      Secondary = true,
      CityUnsheathed = true,
      SheathStateCheckTimerInSeconds = 2
    }
  }
  events = {
  	'ADDON_LOADED',
  	'PLAYER_LOGIN',
  	'PLAYER_REGEN_ENABLED',
  	'LOOT_CLOSED',
  	'AUCTION_HOUSE_CLOSED',
  	'UNIT_SPELLCAST_STOP',
  	'UNIT_SPELLCAST_CHANNEL_STOP',
  	'UNIT_EXITED_VEHICLE',
  	'AUCTION_HOUSE_CLOSED',
  	'BARBER_SHOP_CLOSE',
  	'PLAYER_ENTERING_WORLD',
  	'UNIT_SPELLCAST_SUCCEEDED',
  	'UNIT_AURA',
  	'QUEST_ACCEPTED',
  	'QUEST_FINISHED', 
  	'MERCHANT_CLOSED'
	}
  eventFrame = CreateFrame("FRAME", "StayUnsheathedEventFrame")
  timer = nil
  StayUnsheathed.db = LibStub("AceDB-3.0"):New("StayUnsheathedDB", defaults)
  registerEvents(events)
  registerChatCommands()
  print("StayUnsheathed 1.0 Loaded! Type /su for options.")
end

function registerEvents(events)
  for i = 1, #events do
    eventFrame:RegisterEvent(events[i])
  end
  eventFrame:SetScript("OnEvent", function (frame, event, first, second)
  	  ToggleSheath()
  	end)
end

function registerChatCommands()
  StayUnsheathed:RegisterChatCommand("stayunsheathed","slashfunc")
  StayUnsheathed:RegisterChatCommand("su","slashfunc")
end

function StayUnsheathed:OnEnable()
  startTimer()
  ToggleSheath()
end

function startTimer()
  destroyExistingTimer()
  StayUnsheathed.Timer = StayUnsheathed:ScheduleRepeatingTimer("TimerFeedback", StayUnsheathed.db.char.SheathStateCheckTimerInSeconds)
end

function destroyExistingTimer()
	if (not (StayUnsheathed.Timer == nil)) then
  	StayUnsheathed:CancelTimer(StayUnsheathed.Timer)
  end
end

function StayUnsheathed:TimerFeedback()
  if (isSheathed()) then
    unsheathWeapon()
  end
end

function isSheathed()
  sheathed = 1
  if(GetSheathState() == sheathed) then
    return true
  else
    return false
  end
end

function unsheathWeapon()
  if sheathOptionsAreMet() then
	ToggleSheath()
  end
end

function sheathOptionsAreMet()
	return isStayUnsheathedEnabled() and isSpecEnabled() and isCityEnabled() and not InCombatLockdown()
end

function isStayUnsheathedEnabled()
  return StayUnsheathed.db.char.EnabledState 
end

function isSpecEnabled()
  if GetActiveSpecGroup() == 1 then
  return StayUnsheathed.db.char.Primary
  end
  if GetActiveSpecGroup() == 2 then
  return StayUnsheathed.db.char.Secondary
  end
end

function isCityEnabled()
  if IsResting() then
    return StayUnsheathed.db.char.CityUnsheathed
  else
    return true
  end
end

function StayUnsheathed:slashfunc(input)
	if input == "" or input == "help" then
	  printOptionMenu()
	  printStatus()

	elseif input == "enable" then
	  enableStayUnsheathed()

	elseif input == "disable" then
	  disableStayUnsheathed()

	elseif input == "primary" then
	  togglePrimary()
	  print("Primary spec is now " .. booleanToString(getPrimaryState()))

	elseif input == "secondary" then
	  toggleSecondary()
	  print("Secondary spec is now " .. booleanToString(getSecondaryState()))

	elseif input == "togglecity" then
	  toggleCity()

	elseif input == "info" then
	  printStatus()

	elseif getSlashCommandWithoutParameters(input) == "setCheckTimer" then
	  setCheckTimer(input)
	end
end

function printOptionMenu()
  print("StayUnsheathed options menu.")
  print("Avaliable options")
  print("/su Enable (Enables the addon).")
  print("/su Disable (Disables the addon)") 
  print("/su Primary (Disables/Enables the addon in your primary spec.)")
  print("/su Secondary (Disables/Enables the addon in your secondary spec.)")
  print("/su info (Prints current settings)")
  print("/su togglecity (Toggles Sheathing in Cities)")
  print("/su setCheckTimer x (Sets the check for sheathed weapons to x seconds")
  print("     x must be greater than 0)")
end

function printStatus()
  print("StayUnsheathed is: " .. booleanToString(StayUnsheathed.db.char.EnabledState))
  print("Primary spec is: " .. booleanToString(getPrimaryState()))
  print("Secondary spec is: " .. booleanToString(getSecondaryState()))
  print("You are staying " .. sheathStateToString(getCityUnsheathed()) .. " in cities")
  print("Checking sheath state every " .. StayUnsheathed.db.char.SheathStateCheckTimerInSeconds .. " seconds.")
end
	
function setCheckTimer(input)
	command, argument = StayUnsheathed:GetArgs(input, 2)
	newWaitTime = tonumber(argument)
	minimumWaitTime = 1
	if (newWaitTime >= minimumWaitTime) then
	  StayUnsheathed.db.char.SheathStateCheckTimerInSeconds = newWaitTime
	  startTimer()
	  print("CheckTimer set to " .. StayUnsheathed.db.char.SheathStateCheckTimerInSeconds .. " seconds.")
	else
	  print("CheckTimer requires a number greater than " .. minimumWaitTime .. " seconds.")
	end
end

function getPrimaryState()
  return StayUnsheathed.db.char.Primary
end

function getSecondaryState()
  return StayUnsheathed.db.char.Secondary
end

function getCityUnsheathed()
  return StayUnsheathed.db.char.CityUnsheathed
end

function sheathStateToString(unsheathed)
  if unsheathed == true then 
    return "Unsheathed"
  else 
    return "Sheathed" 
  end
end

function togglePrimary()
	if (getPrimaryState() == true) then
	  StayUnsheathed.db.char.Primary = false
	else
	  StayUnsheathed.db.char.Primary = true
	end
end

function toggleSecondary()
	if (getSecondaryState() == true) then
	  StayUnsheathed.db.char.Secondary = false
	else
	  StayUnsheathed.db.char.Secondary = true
	end
end

function toggleCity()
  if StayUnsheathed.db.char.CityUnsheathed then
    StayUnsheathed.db.char.CityUnsheathed = false
  else
    StayUnsheathed.db.char.CityUnsheathed = true
  end
  printStatus()
end

function toggleEnable()
  if isStayUnsheathedEnabled() then
    disableStayUnsheathed()
  else
    enableStayUnsheathed()
  end
end

function enableStayUnsheathed()
  StayUnsheathed.db.char.EnabledState = true
  startTimer()
  printStatus()
end

function disableStayUnsheathed()
  StayUnsheathed.db.char.EnabledState = false
  destroyExistingTimer()
  printStatus()
end

function getSlashCommandWithoutParameters(input)
  command, argument = StayUnsheathed:GetArgs(input, 2)
  return command
end

function booleanToString(boolean)
   if boolean == true then 
     return "Enabled" 
   else 
     return "Disabled" 
   end
end